<?php

namespace Drupal\alter_status_message\EventSubscriber;

use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpKernel\KernelEvents;
use Symfony\Component\HttpKernel\Event\ResponseEvent;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Messenger\MessengerTrait;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Alters status messages globally based on configured overrides.
 */
class StatusMessageAlterSubscriber implements EventSubscriberInterface {
  use MessengerTrait;

  /**
   * The config factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs the subscriber.
   */
  public function __construct(ConfigFactoryInterface $config_factory) {
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents() {
    return [
      KernelEvents::RESPONSE => ['onRespond', 100],
    ];
  }

  /**
   * Container factory method.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory')
    );
  }

  /**
   * Respond event handler: override status messages.
   */
  public function onRespond(ResponseEvent $event) {
    $messages = $this->messenger()->all();

    if (!empty($messages)) {
      $this->messenger()->deleteAll();

      $config = $this->configFactory->get('alter_status_message.settings');
      $overrides_raw = $config->get('message_overrides') ?? '';

      // Parse overrides into an associative array.
      $overrides = [];
      $lines = preg_split('/\r\n|\r|\n/', $overrides_raw);
      foreach ($lines as $line) {
        if (strpos($line, '=>') !== FALSE) {
          [$original, $replacement] = explode('=>', $line, 2);
          $original = trim($original);
          $replacement = trim($replacement);
          if ($original !== '') {
            $overrides[$original] = $replacement;
          }
        }
      }

      // Iterate and replace messages.
      foreach ($messages as $type => $messages_by_type) {
        foreach ($messages_by_type as $message) {
          $replaced = FALSE;
          foreach ($overrides as $pattern => $replacement) {
            // Check if the pattern contains a [name] token.
            if (strpos($pattern, '[name]') !== FALSE) {
              // Convert [name] to a regex capture group.
              $regex = preg_quote($pattern, '/');
              $regex = str_replace('\[name\]', '(.+)', $regex);
              $regex = '/^' . $regex . '$/';
              if (preg_match($regex, $message, $matches)) {
                // Replace [name] token in the replacement.
                $final_message = str_replace('[name]', $matches[1], $replacement);
                $this->messenger()->addMessage($final_message, $type);
                $replaced = TRUE;
                break;
              }
            }
            else {
              // No token, do simple string match.
              if (trim($pattern) !== '' && strpos($message, $pattern) !== FALSE) {
                $this->messenger()->addMessage($replacement, $type);
                $replaced = TRUE;
                break;
              }
            }
          }
          // If no override matched, keep original.
          if (!$replaced) {
            $this->messenger()->addMessage($message, $type);
          }
        }
      }
    }
  }

}
